@@ -17,9 +17,6 @@ Part of the code was imported from Gaudi and inspired by
 * https://github.com/HEP-SF/documents/tree/master/HSF-TN/draft-2015-NAM
 * https://github.com/HEP-SF/tools
 '''
-import platform
-import sys
-import os
 
 try:
     from pkg_resources import get_distribution, DistributionNotFound
@@ -41,12 +38,8 @@ OS_ALIASES = {
     'ubuntu1704': [],
     'ubuntu1710': [],
     'ubuntu1804': [],
-}
-
-# map reference architectures to equivalent ones
-ARCH_ALIASES = {
-    'i686': ['i386'],
-    'x86_64': ['AMD64'],  # Windows architecture name
+    'ubuntu1810': [],
+    'ubuntu1904': [],
 }
 
 # define compatibility between reference OSs:
@@ -60,29 +53,6 @@ OS_COMPATIBILITY = {
     'slc5': ['slc4'],
 }
 
-# define compatibility between CPU architectures:
-# - the architecture in the key can run binaries built for the architecture in
-#   the value list
-# - compatibility is meant to be transitive (if A can run B, and B can run C,
-#   then A can run C) unless the entry in the list starts with '!' (e.g.
-#   {'A': ['B', '!C'], 'B': ['C']} means that A cannot run C even if B can)
-ARCH_COMPATIBILITY = {
-    'x86_64': ['i686'],
-    'aarch64': []
-}
-
-# known microarchitecture feature in order of inclusion (the 1st is a superset
-# of the 2nd, etc.)
-MICROARCH_LEVELS = ['avx2+fma', 'avx2', 'avx', 'sse4_2']
-
-# mapping between known binary tags (i.e. CMTCONFIG values) and minimum
-# required platform (only special cases, the default is the second component
-# of the BINARY_TAG)
-BINARY_TAG_REQUIREMENTS = {
-    'x86_64-centos7-gcc62': 'x86_64-centos7.sse4_2',
-    'x86_64-slc6-gcc62': 'x86_64-slc6.sse4_2',
-}
-
 
 def normal_name(name, aliases, default=None):
     '''
@@ -136,53 +106,51 @@ def dirac_platform(force_os=None):
 
     If force_os is specified use that instead of detecting it.
     '''
-    from LbPlatformUtils.inspect import architecture, os_id, microarch_flags
+    from LbPlatformUtils.inspect import architecture, os_id
     arch = architecture()
     short_id = normal_name(force_os or os_id(), OS_ALIASES, 'unknown')
+    return '-'.join([arch, short_id])
 
-    microarch = ''
-    flags = microarch_flags()
-    for f in MICROARCH_LEVELS:
-        if flags.issuperset(f.split('+')):
-            microarch = f
-            break
 
-    return arch + '-' + '.'.join(bit for bit in [short_id, microarch] if bit)
+def _promote_old_style_arch(arch):
+    if '+' in arch:
+        from LbPlatformUtils.architectures import (ARCH_DEFS,
+                                                   get_compatible_archs)
+        arch, flags = arch.split('+', 1)
+        flags = set(flags.split('+'))
+        flags.update(ARCH_DEFS.get(arch, []))
+        # take the last compatible arch
+        for arch in get_compatible_archs(flags):
+            pass
+    return arch
 
 
 def requires(binary_tag):
     '''
     Return the minimum required platform for a binary_tag.
     '''
-    binary_tag = binary_tag.split('-')
-    if '+' in binary_tag[0]:
-        # explicit microarch
-        arch, microarch = binary_tag[0].split('+', 1)
-        return '%s-%s.%s' % (arch, binary_tag[1], microarch)
-    # we try to match the prefix of the binary_tag
-    for sub in ['-'.join(binary_tag[:n])
-                for n in range(len(binary_tag), 0, -1)]:
-        if sub in BINARY_TAG_REQUIREMENTS:
-            return BINARY_TAG_REQUIREMENTS[sub]
-    return '-'.join(binary_tag[0:2])  # require plain arch.OS
+    arch, os_id, comp = binary_tag.split('-')[:3]
+    if comp.startswith('gcc') and comp >= 'gcc6':
+        arch += '+sse4_2'
+    arch = _promote_old_style_arch(arch)
+    return '-'.join([arch, os_id])
 
 
-def _split_platform(platf):
+def _parse_platform(platf):
     '''
     Split a dirac platform in a tuple (os, microarch), where microarch is None
     if not specified.
 
-    >>> _split_platform('i686-slc5')
-    ('i686', 'slc5', None)
-    >>> _split_platform('x86_64-centos7.avx2')
-    ('x86_64', 'centos7', 'avx2')
+    >>> _parse_platform('i686-slc5')
+    ('i686', 'slc5')
+    >>> _parse_platform('x86_64-centos7.avx2')
+    ('haswell', 'centos7')
     '''
-    arch, rest = platf.split('-', 1)
-    if '.' in rest:
-        os, micro = rest.split('.', 1)
-    else:
-        os, micro = rest, None
-    return (arch, os, micro)
+    arch, os_id = platf.split('-', 1)
+    if '.' in os_id:
+        os_id, micro = os_id.split('.', 1)
+        arch = _promote_old_style_arch(arch + '+' + micro)
+    return (arch, os_id)
 
 
 def check_compatibility(a, b, compatibility_map):
@@ -209,10 +177,11 @@ def check_compatibility(a, b, compatibility_map):
         allowed = compatibility_map.get(a, [])
         if '!' + b in allowed:  # explicitely disallowed
             return False
-        if b not in allowed:  # not explicitely allower, recurse
+        if b not in allowed:  # not explicitely allowed, recurse
             fallback = [alt for alt in allowed if not alt.startswith('!')]
-            if not any(check_compatibility(f, b, compatibility_map)
-                       for f in fallback):
+            if not any(
+                    check_compatibility(f, b, compatibility_map)
+                    for f in fallback):
                 return False
     return True
 
@@ -221,27 +190,74 @@ def can_run(current, required):
     '''
     Tell if the current platform meets the constraints of the required one.
     '''
-    current_arch, current_os, current_microarch = _split_platform(current)
-    required_arch, required_os, required_microarch = _split_platform(required)
+    current_arch, current_os = _parse_platform(current)
+    required_arch, required_os = _parse_platform(required)
     if 'unknown' in (current_os, required_os, current_arch, required_arch):
         return False  # we do not have enough information
-    # check arch and os
-    if not (check_compatibility(current_arch, required_arch,
-                                ARCH_COMPATIBILITY) and
-            check_compatibility(current_os, required_os,
-                                OS_COMPATIBILITY)):
+    # check os
+    if not check_compatibility(current_os, required_os, OS_COMPATIBILITY):
         return False
-    # check microarch
-    # (note: if nothing is required, anything fits)
-    if required_microarch is None or required_microarch == current_microarch:
-        return True
-    if required_microarch not in MICROARCH_LEVELS:
-        return False  # we do not know anything about the required microarch
-    if current_microarch not in MICROARCH_LEVELS:
-        # at this point, required is something we know (and not None), so,
-        # if current is not known or None, it's not compatible
-        return False
-    # what remains to check is if current includes required, according to the
-    # order in MICROARCH_LEVELS
-    return (MICROARCH_LEVELS.index(current_microarch) <=
-            MICROARCH_LEVELS.index(required_microarch))
+    # check arch
+    from LbPlatformUtils.architectures import ARCH_DEFS
+    return (current_arch == required_arch
+            or ARCH_DEFS.get(current_arch, set()).issuperset(
+                ARCH_DEFS.get(required_arch, set(['unknown']))))
+
+
+def compatible_platforms(required):
+    '''
+    List the dirac platforms that match the required platform.
+
+    >>> compatible_platforms('skylake-centos7')
+    ['cannonlake-centos7', 'skylake_avx512-centos7', 'skylake-centos7']
+    >>> compatible_platforms('unkown-slc6')
+    []
+    '''
+    from LbPlatformUtils.architectures import get_compatible_archs, ARCH_DEFS
+    from re import findall
+    required_arch, required_os = _parse_platform(required)
+    # we cannot say anything about unkown OSs or architectures
+    if ('unknown' in (required_arch, required_os)
+            or required_arch not in ARCH_DEFS):
+        return []
+    # get the list of all archs that are compatible
+    archs = list(get_compatible_archs(ARCH_DEFS[required_arch]))
+    # get the list of all compatible OSs
+    # - first get all known OSs (those from compatibility list plus
+    #   the required one)
+    all_OSs = set(OS_COMPATIBILITY)
+    all_OSs.add(required_os)
+    all_OSs = sorted(all_OSs,key=lambda s: map(int, findall(r'\d+', s)), reverse=True)
+    # - filter by compatibility
+    os_ids = [
+        os_id for os_id in all_OSs
+        if check_compatibility(os_id, required_os, OS_COMPATIBILITY)
+    ]
+    # combinatorics
+    return [
+        '{0}-{1}'.format(arch, os_id) for os_id in os_ids for arch in archs
+    ]
+
+
+def lowest_common_requirement(configs):
+    '''
+    Given a collection of LHCb platform ids, return the minimum required dirac
+    platform that can run all of them, returning None if it cannot be found.
+
+    >>> print(lowest_common_requirement(['x86_64+avx2+fma-slc6-gcc7-opt',
+    ...                                  'x86_64-centos7-gcc62-opt']))
+    haswell-centos7
+    >>> print(lowest_common_requirement(['x86_64-slc5-gcc49-opt',
+    ...                                 'x86_64-centos7-gcc8-opt']))
+    None
+    '''
+    what = None
+    for c in configs:
+        if what is None:
+            what = compatible_platforms(requires(c))
+        else:
+            what = [p for p in compatible_platforms(requires(c)) if p in what]
+    if what:
+        return what[-1]
+    else:
+        return None
